<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * WorthstockReceiver è la classe che attiva la route API di Wordpress in ascolto per le chiamate Webhook da WORTHSTOCK di aggiornamento quantità per i prodotti
 * L'url di Webhook viene comunicato nella chiamata di login alle API remote.
 * La route è pubblica e aperta, ma contiene un meccanismo di riconoscimento basato sul TOKEN di autenticazione alle API remote.
 * 
 * La classe contiene anche una serie di route di test interrogabili dietro autenticazione
 */
class WorthstockReceiver extends WorthstockBase {

    const PLUGIN_API_BASE_URL = "worthstock";
    const PLUGIN_API_VERSION = 1;
    const WP_API_PREFIX = "wp-json";
    const WS_SALT = '!?W0rt45T0ck?!';

    public function __construct()
    {
        add_action( 'rest_api_init', array($this, 'register_api_routes'), 10 );
    }

    public function register_api_routes() {
        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            // 'worthstock',
            //'/updateproductstock/(?P<id>\d+)',
            '/updateproductstock/',
            array(
                //'methods' => WP_REST_Server::EDITABLE, // PUT, POST, PATCH
                'methods' => WP_REST_Server::ALLMETHODS,
                //'methods' => 'GET',
                'callback' => array($this, 'localUpdateProductStock'),
                'permission_callback' => '__return_true', // public
            )
        );

        
        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/get_product/(?P<id>\d+)',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'getMappedProduct'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );

        
        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/notify_price/(?P<id>\d+)',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'testNotifyPriceUpdate'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );
        
        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/notify_quantity/(?P<id>\d+)',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'testNotifyQuantityUpdate'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );

        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/notify_delete/(?P<id>\d+)',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'testNotifyDelete'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );

        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/notify_update/(?P<id>\d+)',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'testNotifyUpdate'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );

        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/notify_insert/(?P<id>\d+)',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'testNotifyInsert'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );

        register_rest_route(
            self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '',
            '/publish_endpoint/',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'testPrintEndpoint'),
                'permission_callback' => array($this, 'apiAuthCallback')
            )
        );
    }

    public function apiAuthCallback($request) {
        return current_user_can('manage_woocommerce');
    }

    /**
     * REST API callback
     */
    public function localUpdateProductStock($request) {
        //$param = $request->get_params();
        //$body = $param = $request->get_body();
        $postparam = $request->get_params();
        $body = $request->get_body();
        $remote_method = $request->get_method();
        if (is_string($body)) {
            $parameters = json_decode($body, true);
        }
        $referer = $request->get_header( 'referer' );
        $received_token =  trim($request->get_header( 'X-PIC' ));
        //$received_token = hash("sha256", wp_unslash('{"Products":[{"IdProduct":"489","Variants":[{"IdVariant":"4762","UniqueIdShopify":"gid:\/\/shopify\/ProductVariant\/39736941871293","Quantity":-1}]}]}') . self::WS_SALT);
        $this->main()->sync->api_logger('Salted hash ' . $received_token);
        $this->main()->sync->api_logger('Hashed request ' . hash("sha256", wp_unslash($request->get_body()) . self::WS_SALT));

        //if ( !$this->apiCallAuthorizationCheck($request) && false) {
        if ( !$this->apiCallAuthorizationCheck($request) ) {
            $this->main()->sync->api_logger('Chiamata webhook non autorizzata');
            wp_send_json_error(['message'=>'Forbidden'], 403);   
        }

        $host = wp_parse_url( $referer, PHP_URL_HOST );
        $remote_ip = $_SERVER['REMOTE_ADDR'];

        $this->main()->sync->api_logger("Chiamata" .($remote_method ? ' ' . $remote_method : '') . " ricevuta da" .($referer ? ' ' . $referer : '') . ' ' .($remote_ip ? ' '.$remote_ip : '') . ($parameters ? " parsed params " .  json_encode($parameters) : '') . ($postparam ? " postparams " .  json_encode($postparam) : ''));
        $this->main()->sync->api_logger('Body' . json_encode($body));
        //print_r($request);
        
        //disabilitiamo gli aggiornamenti verso worthstock per evitare un loop
        $this->main()->set_woocommerce_replication(false);
        // print_r($parameters);
        // exit();
        //exit(json_encode($body));
        //wp_send_json_success($parameters, 200);
        if ( ($products = $this->checkRequestParams($parameters['Products'])) === false) {
            wp_send_json_error(['message'=>'malformed request'], 400);            
        }
        $processed_variations = [];
        foreach($parameters['Products'] as $product) {
            $id_product = $product['IdProduct'];
            foreach($product['Variants'] as $variant) {
                $id_variation = $variant['IdVariant'];
                $quantity_variation = $variant['Quantity'];
                /* if(!$id_product || !$id_variation || !$quantity_variation) {
                    wp_send_json_error(['message'=>'malformed request'], 400);
                } */
                
                $message = "Chiamata webhook ricevuta da Worthstock: (IdProduct ".$id_product.", IdVariant  ".$id_variation.", Quantità ".$quantity_variation.")";
                $this->main()->sync->api_logger($message);
                $result = $this->updateProductStock($id_product, $id_variation, $quantity_variation);
                if (!$result) {
                    wp_send_json_error(['message'=>'internal error'], 500);
                }
            }
        }
        $processed_variations = $product['Variants'];
        
        wp_send_json_success($parameters, 200);
        //wp_send_json_success([$id_product, $processed_variations], 200);
        wp_die();
        // $request->get_params();
        // $request->get_body();
        // Do stuff        
        return new WP_REST_Response($request, 200);
        //wp_die('hello');
    }

    protected function apiCallAuthorizationCheck($request) {
        $received_token =  $request->get_header( 'X-PIC' );
        $processed_token = hash("sha256", wp_unslash($request->get_body()) . self::WS_SALT);
        if ($received_token == $processed_token) {
            return true;
        }
        return false;
    }

    protected function updateProductStock($id_product, $id_variation, $quantity_variation) {
        $product_id = $this->main()->sync->getProductByWorthstockId($id_product);
        $variation_id = $this->main()->sync->getVariantByWorthstockId($id_variation);
        $product = wc_get_product( $product_id );
        /* if ( $product->is_type('simple') && !$variation_id) {
            $variation_id = true;
        }
        if ( !$product_id || !$variation_id || !$product) return false; */
        if ($quantity_variation < 1) {
            $stock_action= 'decrease';
        } else {
            $stock_action= 'increase';
        }
        $quantity_variation = abs(intval($quantity_variation));
        
        // se non gestisce magazzino non aggiorno nulla e restituisco successo
        if ( $product->managing_stock() === false && !$variation_id ) {
            $this->main()->set_woocommerce_replication(true);
            $response = $this->main()->sync->notify_stock_updated($product_id);
            $this->main()->set_woocommerce_replication(false);
            return true;
        } else if ( $product->is_type('variable') ) {
            $status = wc_update_product_stock( $variation_id, $quantity_variation, $stock_action );
        } else {
            $status = wc_update_product_stock( $product_id, $quantity_variation, $stock_action );
        }
        $status = true;
        if ($status) {
            wc_delete_product_transients($product_id); // Clear/refresh the variation cache (optionally if needed)
            wc_delete_product_transients($variation_id); // Clear/refresh the variation cache (optionally if needed)
            $this->main()->sync->save_product_last_operation('Stock aggiornato da Worthstock', 'success', $product_id);
            $message = "Quantità stock aggiornata per (".$product_id.")";
        } else {
            $this->main()->sync->save_product_last_operation('Stock aggiornato da Worthstock', 'failure', $product_id);
            $message = "Quantità stock non aggiornata per (".$product_id.")";
        }
        $this->main()->sync->api_logger($message);
        return $status;
    }

    
    protected function checkRequestParams($products_data) {    
        foreach($products_data as $product) {
            $id_product = $product['IdProduct'];
            if ( !$id_product ) return false;
            $product_id = $this->main()->sync->getProductByWorthstockId($id_product);
            if ( !$product_id ) return false;
            $product_object = wc_get_product( $product_id );
            if ( !$id_product || !$product_id || !$product) {return false;}
            if ( !isset($product['Variants']) || empty($product['Variants']) ) { return false; }

            foreach($product['Variants'] as $variant) {
                $id_variation = $variant['IdVariant'];
                $quantity_variation = $variant['Quantity'];
                $variation_id = $this->main()->sync->getVariantByWorthstockId($id_variation);
                if ( $product_object->is_type('simple') && !$variation_id) {
                    continue;
                }
                if( !$variation_id || !$id_variation || !abs(intval($quantity_variation)) ) {
                    return false;
                }
            }
        }

        return true;
    }

    public function getMappedProduct ($request) {
        $product_id = $request->get_params()['id'];

        $data = $this->main()->mapper->convertProductToWorthstock($product_id, 'full');
        $data = $this->main()->sync->prepare_product_json('insert', $data);
        return new WP_REST_Response($data, 200);
    }

    public function testNotifyQuantityUpdate ($request) {
        $product_id = $request->get_params()['id'];
        //return new WP_REST_Response($product_id, 200);

        //$data = $this->main()->mapper->convertProductToWorthstock($product_id, 'quantity');
        $response = $this->main()->sync->notify_stock_updated($product_id);
        return new WP_REST_Response($response, 200);
    }

    public function testNotifyPriceUpdate ($request) {
        $product_id = $request->get_params()['id'];
        //return new WP_REST_Response($product_id, 200);

        //$data = $this->main()->mapper->convertProductToWorthstock($product_id, 'quantity');
        $response = $this->main()->sync->notify_price_updated($product_id);
        return new WP_REST_Response($response, 200);
    }
    
    public function testNotifyUpdate ($request) {
        $product_id = $request->get_params()['id'];

        $response = $this->main()->sync->notify_product_updated($product_id);
        return new WP_REST_Response($response, 200);
    }    
    
    public function testNotifyInsert ($request) {
        $product_id = $request->get_params()['id'];

        $response = $this->main()->sync->notify_product_inserted($product_id);
        return new WP_REST_Response($response, 200);
    }
    
    public function testNotifyDelete ($request) {
        $product_id = $request->get_params()['id'];

        $response = $this->main()->sync->notify_product_deleted($product_id);
        return new WP_REST_Response($response, 200);
    }
    
    public function testPrintEndpoint() {
        //exit('HERE I AM');
        $endpoint = $this->returnUpdateProductStockEndpoint();
        return new WP_REST_Response(["endpoint"=>$endpoint], 200);
    }
     

    /**
     * ricostruisce rotta per il webhook da restituire a Worthstock per gli update
     */
    public function returnUpdateProductStockEndpoint() {
        return str_replace('http://www.worthstock.local/','https://worthstock.nextre.it/',site_url() . '/' . self::WP_API_PREFIX . '/' . self::PLUGIN_API_BASE_URL . '/v' . self::PLUGIN_API_VERSION . '/updateproductstock/');
    }

    protected function getSignFromInt($n) {
        return ($n > 0) - ($n < 0);
    }
}