<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * WorthstockSettings è la classe che gestisce visualizzazione e salvataggio della pagina delle opzioni del plugin
 */
class WorthstockSettings extends WorthstockBase {
	private $hook;
	public $slug = 'worthstock-settings';
	protected $options;
	
	public function __construct() {
		add_action( 'init', array( &$this, 'init' ) );
		//add_action( 'admin_menu', array( &$this, 'action_admin_menu' ), 30 );
		add_action( 'admin_init', array( &$this, 'register_settings' ) );
		//add_action( 'admin_notices', array( &$this, 'admin_notices' ) );
		add_action( 'admin_footer', array( &$this, 'admin_footer' ) );
		
		// filtro le options in lettura e scrittura per la crittazione delle password
		add_filter( 'option_worthstock-settings', function( $options ) {
			$crypt = new Worthstock\Google\Site_Kit\Core\Storage\Data_Encryption();
			if ( isset($options['api_secret']) && $options['api_secret'] != '' ) {
				$options['api_secret'] = $crypt->decrypt($options['api_secret']);
			}
			if ( isset($options['api_test_secret']) && $options['api_test_secret'] != '' ) {
				$options['api_test_secret'] = $crypt->decrypt($options['api_test_secret']);
			}
			return $options;
		});
		add_filter( 'pre_update_option_worthstock-settings', function( $new_value, $old_value ) {
			$crypt = new Worthstock\Google\Site_Kit\Core\Storage\Data_Encryption();
			//Do something before returning the new value to be saved in database
			if ( isset($new_value['api_secret']) && $new_value['api_secret'] != '' ) {
				$new_value['api_secret'] = $crypt->encrypt($new_value['api_secret']);
			}
			if ( isset($new_value['api_test_secret']) && $new_value['api_test_secret'] != '' ) {
				$new_value['api_test_secret'] = $crypt->encrypt($new_value['api_test_secret']);
			}
		 
			return $new_value;
		 
		 }, 10, 2);
		//add_action( 'update_option_worthstock-settings', array( &$this, 'updated_option' ), 10, 3 );
		//add_filter( 'plugin_action_links_' . ACTIVITY_LOG_BASE, array( &$this, 'plugin_action_links' ) );

		//add_action( 'wp_ajax_worthstock_reset_items', array( &$this, 'ajax_worthstock_reset_items' ) );
		//add_action( 'wp_ajax_worthstock_get_properties', array( &$this, 'ajax_worthstock_get_properties' ) );
	}
	
	public function init() {
		$this->options = $this->get_options();
	}
	
	public function plugin_action_links( $links ) {
		$settings_link = sprintf( '<a href="%s" target="_blank">%s</a>', 'https://www.worthstock.com/' );
		array_unshift( $links, $settings_link );
		
		$settings_link = sprintf( '<a href="%s">%s</a>', admin_url( 'admin.php?page=activity-log-settings' ), __( 'Settings', 'worthstock' ) );
		array_unshift( $links, $settings_link );
		
		return $links;
	}

	/**
	 * Register the settings page
	 *
	 * @since 1.0
	 */
	/* public function action_admin_menu() {
		$this->hook = add_submenu_page(
			'worthstock',
			__( 'Worthstock plugin Settings', 'worthstock' ), 	// <title> tag
			__( 'Configure', 'worthstock' ), 			// menu label
			'manage_options', 								// required cap to view this page
			$this->slug, 			// page slug
			array( &$this, 'display_settings_page' ),			// callback
			2
		);

		// register scripts & styles, specific for the settings page
		add_action( "admin_print_scripts-{$this->hook}", array( &$this, 'enqueue_scripts' ) );
		// this callback will initialize the settings for worthstock
		// add_action( "load-$this->hook", array( $this, 'register_settings' ) );
	} */

	/**
	 * Register scripts & styles
	 *
	 * @since 1.0
	 */
	public function enqueue_scripts() {

    }

	public function register_settings() {
		// If no options exist, create them.
		if ( !$this->options ) {
		//if ( ! get_option( $this->slug ) ) {
			update_option( $this->slug, apply_filters( 'worthstock_default_options', array(
				'worthstock_enable_replication' => 'yes',
				'worthstock_automatic_replication' => 'no',
			) ) );
		} else {
			// if missing parameter set it to yes
			if ( !$this->options['worthstock_enable_replication'] ) {
				$this->options['worthstock_enable_replication'] = 'yes';
				update_option( $this->slug, apply_filters( 'worthstock_default_options', $this->options) );				
			}
		}

		register_setting( 'worthstock-settings', $this->slug, array( $this, 'validate_options' ) );
		$section = $this->get_setup_section();
         
		switch ( $section ) {
			case 'general':
				// First, we register a section. This is necessary since all future options must belong to a
				add_settings_section(
					'general_settings_section',			// ID used to identify this section and with which to register options
					__( 'Replica', 'worthstock' ),	// Title to be displayed on the administration page
					array( 'worthstock_Settings_Fields', 'general_settings_section_header' ),	// Callback used to render the description of the section
					$this->slug		// Page on which to add this section of options
				);

				add_settings_field(
					'automatic_replication',
					__( 'Sincronizzazione automatica dei dati', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'select_field' ),
					$this->slug,
					'general_settings_section',
					array(
						'id'      => 'worthstock_automatic_replication',
						'page'    => $this->slug,
						//'classes' => array( 'wide-text' ),
						'type'    => 'select',
						'options' => array(
							'yes' => __( 'Sì', 'worthstock' ),
							'no' => __( "No", 'worthstock' ),
						),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Selezionando “Sì” tutti i prodotti sincronizzati con
						WORTHSTOCK saranno aggiornati in automatico dopo ogni modifica effettuata alle schede
						prodotto in WooCommerce', 'worthstock' ),
					)
				);

				add_settings_field(
					'activate_worthstock',
					__( 'Consenti Sincronizzazione', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'select_field' ),
					$this->slug,
					'general_settings_section',
					array(
						'id'      => 'worthstock_enable_replication',
						'page'    => $this->slug,
						//'classes' => array( 'wide-text' ),
						'type'    => 'select',
						'options' => array(
							'yes' => __( 'Sì', 'worthstock' ),
							'no' => __( "No", 'worthstock' ),
						),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Selezionando “No” si disattiva temporaneamente ogni sincronizzazione tra WooCommerce e WORTHSTOCK (se non si è sicuri lasciare impostato a “Sì”)', 'worthstock' ),
					)
				);
				break;
				
			case 'authentication':
				add_settings_section(
					'authentication_settings_section',			// ID used to identify this section and with which to register options
					__( 'Credenziali', 'worthstock' ),	// Title to be displayed on the administration page
					array( 'worthstock_Settings_Fields', 'auth_settings_section_header' ),	// Callback used to render the description of the section
					$this->slug		// Page on which to add this section of options
				);

				add_settings_field(
					'api_test_environment',
					__( 'Utilizzare ambiente di produzione', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'select_field' ),
					$this->slug,
					'authentication_settings_section',
					array(
						'id'      => 'worthstock_test_environment_flag',
						'page'    => $this->slug,
						'type'    => 'checkbox',
						'options' => array(
							'no' => __( "Sì", 'worthstock' ),
							'yes' => __( "No", 'worthstock' ),
						),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Selezionando "No" si utilizzerà l\'ambiente di test', 'worthstock' ),
					)
				);

				add_settings_field(
					'api_login',
					__( 'Username', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'text_field' ),
					$this->slug,
					'authentication_settings_section',
					array(
						'id'      => 'api_login',
						'page'    => $this->slug,
						'classes' => array( 'short-text' ),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Username di connessione', 'worthstock' ),
					)
				);
				add_settings_field(
					'api_secret',
					__( 'Password', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'password_field' ),
					$this->slug,
					'authentication_settings_section',
					array(
						'id'      => 'api_secret',
						'page'    => $this->slug,
						//'classes' => array( 'small-text' ),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Password di connessione', 'worthstock' ),
					)
				);
				
				add_settings_field(
					'api_test_login',
					__( 'Username (test)', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'text_field' ),
					$this->slug,
					'authentication_settings_section',
					array(
						'id'      => 'api_test_login',
						'page'    => $this->slug,
						'classes' => array( 'short-text' ),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Username di connessione', 'worthstock' ),
					)
				);
				add_settings_field(
					'api_test_secret',
					__( 'Password (test)', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'password_field' ),
					$this->slug,
					'authentication_settings_section',
					array(
						'id'      => 'api_test_secret',
						'page'    => $this->slug,
						//'classes' => array( 'small-text' ),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Password di connessione', 'worthstock' ),
					)
				);
				break;

			case 'advanced':
				// First, we register a section. This is necessary since all future options must belong to a
				add_settings_section(
					'advanced_settings_section',			// ID used to identify this section and with which to register options
					__( 'Log di Debug', 'worthstock' ),	// Title to be displayed on the administration page
					array( 'worthstock_Settings_Fields', 'advance_settings_section_header' ),	// Callback used to render the description of the section
					$this->slug		// Page on which to add this section of options
				);

				add_settings_field(
					'activate_debug',
					__( 'Attiva debug verboso', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'select_field' ),
					$this->slug,
					'advanced_settings_section',
					array(
						'id'      => 'worthstock_debug_flag',
						'page'    => $this->slug,
						//'classes' => array( 'small-text' ),
						'type'    => 'select',
						'options' => array(
							'no' => __( "No", 'worthstock' ),
							'yes' => __( 'Sì', 'worthstock' ),
						),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Selezionare “Sì” per salvare su WooCommerce
						automaticamente i log di debug estesi', 'worthstock' ),
					)
				);
				break;
			
			case 'disinstall':
				// First, we register a section. This is necessary since all future options must belong to a
				add_settings_section(
					'disinstall_settings_section',			// ID used to identify this section and with which to register options
					__( 'Disinstallazione', 'worthstock' ),	// Title to be displayed on the administration page
					array( 'worthstock_Settings_Fields', 'advance_settings_section_header' ),	// Callback used to render the description of the section
					$this->slug		// Page on which to add this section of options
				);

				add_settings_field(
					'activate_debug',
					__( 'Rimuovere i dati del plugin?', 'worthstock' ),
					array( 'worthstock_Settings_Fields', 'select_field' ),
					$this->slug,
					'disinstall_settings_section',
					array(
						'id'      => 'worthstock_remove_all_data',
						'page'    => $this->slug,
						'type'    => 'select',
						'options' => array(
							'no' => __( "No", 'worthstock' ),
							'yes' => __( 'Sì', 'worthstock' ),
						),
						//'sub_desc'    => __( 'days.', 'worthstock' ),
						'desc'    => __( 'Selezionare “Sì” per rimuovere i dati associati con il plugin, al momento della disinstallazione', 'worthstock' ),
					)
				);
				break;
		}
	}

	/**
	 * Returns the current section within worthstock's setting pages
	 *
	 * @return string
	 */
	public function get_setup_section() {
		if ( isset( $_REQUEST['worthstock_section'] ) )
			return strtolower( $_REQUEST['worthstock_section'] );

		return 'authentication';
	}

	/**
	 * Prints section tabs within the settings area
	 */
	private function menu_print_tabs() {
		$current_section = $this->get_setup_section();
		$sections = array(
			'authentication'       => __( 'Credenziali', 'worthstock' ),
			'general'       => __( 'Sincronizzazione', 'worthstock' ),
			'disinstall'       => __( 'Opzioni disinstallazione', 'worthstock' ),
			'advanced'       => __( 'Opzioni avanzate', 'worthstock' ),
		);

		$enabled_notification_handlers = $this->get_option( 'notification_handlers' );

		// Hide notifications tab if not used before..
		if ( ! empty( $enabled_notification_handlers ) ) {
			$sections['notifications'] = __( 'Notifications', 'worthstock' );
		}

		$sections = apply_filters( 'worthstock_setup_sections', $sections );

		if ( 1 >= count( $sections ) ) {
			return;
		}

		foreach ( $sections as $section_key => $section_caption ) {
			$active = $current_section === $section_key ? 'nav-tab-active' : '';
			$url = add_query_arg( 'worthstock_section', $section_key );
			echo '<a class="nav-tab ' . $active . '" href="' . esc_url( $url ) . '">' . esc_html( $section_caption ) . '</a>';
		}
	}
	
	public function validate_options( $input ) {
		$options = $this->options; // CTX,L1504
		
		// @todo some data validation/sanitization should go here
		$output = apply_filters( 'worthstock_validate_options', $input, $options );
		
		//if (!empty($output['api_login']) && !empty($output['api_secret']) ) {
			if ( $output['worthstock_test_environment_flag'] == 'yes' ) {
				$api_url = WorthstockSync::API_TEST_URL;
				$api_login = $output['api_test_login'];
				$api_secret = $output['api_test_secret'];
			} else {
				$api_url = WorthstockSync::API_LIVE_URL;
				$api_login = $output['api_login'];
				$api_secret = $output['api_secret'];
			}

			
			if (!empty($api_login) && !empty($api_secret) ) {
				$this->validate_login(array("Username"=>$api_login,"Password"=>$api_secret, "ApiUrl"=>$api_url, 
            "WebhookUrl"=> $this->main()->receiver->returnUpdateProductStockEndpoint()));
			}
		//}
		// merge with current settings
		$output = array_merge( $options, $output );
		
		return $output;
	}

	public function validate_login($pars) {
		$login_api_attempt = $this->main()->sync->api_login(false,$pars);
		if ( $login_api_attempt == false ) {
			$message = __('Credenziali non valide. Si prega di controllare di avere uno user e password validi', 'worthstock');
			//exit('errore ' . $message);
			add_settings_error( $this->slug, 'error', $message, 'error');
		}
	}

	public function display_settings_page() {
		?>
		<!-- Create a header in the default WordPress 'wrap' container -->
		<div class="wrap">

			<h1 class="worthstock-page-title"><?php _e( 'Impostazioni', 'worthstock' ); ?></h1>
			<?php settings_errors(); ?>
			<h2 class="nav-tab-wrapper"><?php $this->menu_print_tabs(); ?></h2>
			
			<form method="post" action="options.php">
				<?php
				settings_fields( 'worthstock-settings' );
				do_settings_sections( $this->slug );
				submit_button();
				?>
			</form>
			
		</div><!-- /.wrap -->
		<?php
	}
	
	public function admin_notices() {
		switch ( filter_input( INPUT_GET, 'message' ) ) {
			case 'data_erased':
				printf( '<div class="updated"><p>%s</p></div>', __( 'All activities have been successfully deleted.', 'worthstock' ) );
				break;

			default:
				printf( '<div class="updated"><p>%s</p></div>', __( 'All activities have been successfully deleted.', 'worthstock' ) );
				
				settings_errors( $this->slug );
				break;
		}
	}
	
	public function admin_footer() {
		// TODO: move to a separate file.
		?>
		<script type="text/javascript">
			jQuery( document ).ready( function( $ ) {
				/* $( '#worthstock-delete-log-activities' ).on( 'click', function( e ) {
					if ( ! confirm( '<?php echo __( 'Attention: We are going to DELETE ALL ACTIVITIES from the database. Are you sure you want to do that?', 'worthstock' ); ?>' ) ) {
						e.preventDefault();
					}
				} ); */
			} );
		</script>
		<?php
	}
	
	public function ajax_worthstock_reset_items() {
		if ( ! check_ajax_referer( 'worthstock_reset_items', '_nonce', false ) || ! current_user_can( 'manage_options' ) ) {
			wp_die( __( 'You do not have sufficient permissions to access this page.', 'worthstock' ) );
		}
		
		$this->main()->api->erase_all_items();
		
		wp_redirect( add_query_arg( array(
				'page' => 'activity-log-settings',
				'message' => 'data_erased',
		), admin_url( 'admin.php' ) ) );
		die();
	}

	public function ajax_worthstock_get_properties() {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error();
		}
			
		$action_category = isset( $_REQUEST['action_category'] ) ? $_REQUEST['action_category'] : false;
		
		$options = $this->main()->notifications->get_settings_dropdown_values( $action_category );

		if ( ! empty( $options ) ) {
			wp_send_json_success( $options );
		}

		wp_send_json_error();
	}

	public function get_option( $key = '' ) {
		$settings = $this->get_options();
		return ! empty( $settings[ $key ] ) ? $settings[ $key ] : false;
	}
	
	/**
	 * Returns all options
	 * 
	 * @since 2.0.7
	 * @return array
	 */
	public function get_options() {
		// Allow other plugins to get worthstock's options.
		if ( isset( $this->options ) && is_array( $this->options ) && ! empty( $this->options ) )
			return $this->options;
		
		return apply_filters( 'worthstock_options', get_option( $this->slug, array() ) );
	}
	
	public function slug() {
		return $this->slug;
	}
}

// TODO: Need rewrite this class to useful tool.
final class worthstock_Settings_Fields {

	public static function general_settings_section_header() {
		?>
		<p><?php //_e( 'Opzioni principali.', 'worthstock' ); ?></p>
		<?php
	}

	public static function advance_settings_section_header() {
		?>
		<p><?php //_e( 'Attenzione! Opzioni avanzate', 'worthstock' ); ?></p>
		<?php
	}

	public static function auth_settings_section_header() {
		?>
		<p><?php //_e( 'Credenziali.', 'worthstock' ); ?></p>
		<?php
	}
	
	public static function raw_html( $args ) {
		if ( empty( $args['html'] ) )
			return;
		
		echo $args['html'];
		if ( ! empty( $args['desc'] ) ) : ?>
			<p class="description"><?php echo $args['desc']; ?></p>
		<?php endif;
	}
	
	public static function text_field( $args ) {
		self::_set_name_and_value( $args );
		extract( $args, EXTR_SKIP );
		
		$args = wp_parse_args( $args, array(
			'classes' => array(),
		) );
		if ( empty( $args['id'] ) || empty( $args['page'] ) )
			return;
		
		?>
		<input type="text" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $value ); ?>" class="<?php echo implode( ' ', $args['classes'] ); ?>" />
		<?php if ( ! empty( $desc ) ) : ?>
		<p class="description"><?php echo $desc; ?></p>
		<?php endif;
	}
	
	public static function password_field( $args ) {
		self::_set_name_and_value( $args );
		extract( $args, EXTR_SKIP );
		
		$args = wp_parse_args( $args, array(
			'classes' => array(),
		) );
		if ( empty( $args['id'] ) || empty( $args['page'] ) )
			return;
		
		?>
		<input type="password" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $value ); ?>" class="<?php echo implode( ' ', $args['classes'] ); ?>" autocomplete="new-password" />
		<?php if ( ! empty( $desc ) ) : ?>
		<p class="description"><?php echo $desc; ?></p>
		<?php endif;
	}

	public static function textarea_field( $args ) {
		self::_set_name_and_value( $args );
		extract( $args, EXTR_SKIP );

		$args = wp_parse_args( $args, array(
			'classes' => array(),
			'rows'    => 5,
			'cols'    => 50,
		) );

		if ( empty( $args['id'] ) || empty( $args['page'] ) )
			return;

		?>
		<textarea id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $name ); ?>" class="<?php echo implode( ' ', $args['classes'] ); ?>" rows="<?php echo absint( $args['rows'] ); ?>" cols="<?php echo absint( $args['cols'] ); ?>"><?php echo esc_textarea( $value ); ?></textarea>

		<?php if ( ! empty( $desc ) ) : ?>
			<p class="description"><?php echo $desc; ?></p>
		<?php endif;
	}
	
	public static function number_field( $args ) {
		self::_set_name_and_value( $args );
		extract( $args, EXTR_SKIP );
		
		$args = wp_parse_args( $args, array(
			'classes' => array(),
			'min' => '1',
			'step' => '1',
			'desc' => '',
		) );
		if ( empty( $args['id'] ) || empty( $args['page'] ) )
			return;

		?>
		<input type="number" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $value ); ?>" class="<?php echo implode( ' ', $args['classes'] ); ?>" min="<?php echo $args['min']; ?>" step="<?php echo $args['step']; ?>" />
		<?php if ( ! empty( $args['sub_desc'] ) ) echo $args['sub_desc']; ?>
		<?php if ( ! empty( $args['desc'] ) ) : ?>
			<p class="description"><?php echo $args['desc']; ?></p>
		<?php endif;
	}

	public static function select_field( $args ) {
		self::_set_name_and_value( $args );
		extract( $args, EXTR_SKIP );

		if ( empty( $options ) || empty( $id ) || empty( $page ) )
			return;
		
		?>
		<select id="<?php echo esc_attr( $id ); ?>" name="<?php printf( '%s[%s]', esc_attr( $page ), esc_attr( $id ) ); ?>">
			<?php foreach ( $options as $name => $label ) : ?>
			<option value="<?php echo esc_attr( $name ); ?>" <?php selected( $name, (string) $value ); ?>>
				<?php echo esc_html( $label ); ?>
			</option>
			<?php endforeach; ?>
		</select>
		<?php if ( ! empty( $desc ) ) : ?>
		<p class="description"><?php echo $desc; ?></p>
		<?php endif; ?>
		<?php
	}
	
	public static function yesno_field( $args ) {
		self::_set_name_and_value( $args );
		extract( $args, EXTR_SKIP );
		
		?>
		<label class="tix-yes-no description"><input type="radio" name="<?php echo esc_attr( $name ); ?>" value="1" <?php checked( $value, true ); ?>> <?php _e( 'Yes', 'worthstock' ); ?></label>
		<label class="tix-yes-no description"><input type="radio" name="<?php echo esc_attr( $name ); ?>" value="0" <?php checked( $value, false ); ?>> <?php _e( 'No', 'worthstock' ); ?></label>

		<?php if ( isset( $args['description'] ) ) : ?>
		<p class="description"><?php echo $args['description']; ?></p>
		<?php endif; ?>
		<?php
	}

	public static function email_notification_buffer_field( $args ) {
		$args = wp_parse_args( $args, array(
			'classes' => array(),
		) );
		if ( empty( $args['id'] ) || empty( $args['page'] ) )
			return;

		// available action categories
		$keys = array(
			'user' 			=> __( 'User', 'worthstock' ),
			'action-type' 	=> __( 'Action Type', 'worthstock' ),
			'action-value'  => __( 'Action Performed', 'worthstock' ),
		);
		// available condition types
		$conditions = array(
			'equals' => __( 'equals to', 'worthstock' ),
			'not_equals' => __( 'not equals to', 'worthstock' ),
		);

		$common_name = sprintf( '%s[%s]', esc_attr( $args['page'] ), esc_attr( $args['id'] ) );

		// get all rows
		$rows = Worthstock::instance()->settings->get_option( $args['id'] );
		// if empty, reset to one element with the key of 1
		$rows = empty( $rows ) ? array( array( 'key' => 1 ) ) : $rows;
		?>
		<p class="description"><?php _e( 'A notification will be sent upon a successful match with the following conditions:', 'worthstock' ); ?></p>
		<div class="worthstock-notifier-settings">
			<ul>
			<?php foreach ( $rows as $rid => $row ) :
				$row_key 		= $row['key']; 
				$row_condition 	= isset( $row['condition'] ) ? $row['condition'] : '';
				$row_value 		= isset( $row['value'] ) ? $row['value'] : '';
				?>
				<li data-id="<?php echo $rid; ?>">
					<select name="<?php echo $common_name; ?>[<?php echo $rid; ?>][key]" class="worthstock-category">
						<?php foreach ( $keys as $k => $v ) : ?>
						<option value="<?php echo $k; ?>" <?php selected( $row_key, $k ); ?>><?php echo $v; ?></option>
						<?php endforeach; ?>
					</select>
					<select name="<?php echo $common_name; ?>[<?php echo $rid; ?>][condition]" class="worthstock-condition">
						<?php foreach ( $conditions as $k => $v ) : ?>
						<option value="<?php echo $k; ?>" <?php selected( $row_condition, $k ); ?>><?php echo $v; ?></option>
						<?php endforeach; ?>
					</select>
					<?php $value_options = $this->main()->notifications->get_settings_dropdown_values( $row_key ); ?>
					<select name="<?php echo $common_name; ?>[<?php echo $rid; ?>][value]" class="worthstock-value">
						<?php foreach ( $value_options as $option_key => $option_value ) : ?>
						<option value="<?php echo esc_attr( $option_key ); ?>" <?php selected( $option_key, $row_value ); ?>><?php echo esc_html( $option_value ); ?></option>
						<?php endforeach; ?>
					</select>
					<a href="#" class="worthstock-new-rule button"><small>+</small> <?php _e( 'and', 'worthstock' ); ?></a>
					<a href="#" class="worthstock-delete-rule button">&times;</a>
				</li>
			<?php endforeach; ?>
			</ul>
		</div>
		<?php
	}
	
	private static function _set_name_and_value( &$args ) {
        //return;
		if ( ! isset( $args['name'] ) ) {
			$args['name'] = sprintf( '%s[%s]', esc_attr( $args['page'] ), esc_attr( $args['id'] ) );
		}
		
		if ( ! isset( $args['value'] ) ) {
			$args['value'] = Worthstock::instance()->settings->get_option( $args['id'] );
		}
	}
}
