<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * WorthstockUpdater è la classe che si occupa di controllare se ci sono nuove versioni disponibili del plugin (ospitate nel dominio di worthstock.com) e permettere all'utente di effettuare l'aggiornamento, grazie all'apposita API di Wordpress
 */
if( ! class_exists( 'worthstockUpdater' ) ) {

    class WorthstockUpdater extends WorthstockBase {

        public $plugin_slug;
        public $plugin_filename;
        public $plugin_sitelocation;
        public $version;
        public $cache_key;
        public $cache_allowed;

        public function __construct() {

            //$this->plugin_slug = worthstock::instance()->plugin_slug;
            $this->plugin_slug = 'worthstock';
            $this->plugin_filename =  'worthstock/worthstock.php';
            //$this->plugin_sitelocation =  'https://worthstock.nextre.it/wp-content/uploads/worthstock-updater/';
            $this->plugin_sitelocation =  'https://worthstock.com/woocommerce-plugin-updater/';            
            $this->version =  worthstock::$plugin_version;
            $this->cache_key = 'worthstock_custom_upd';
            $this->cache_allowed = true;

            add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
            add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );
            add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );

        }

        public function request(){

            $remote = get_transient( $this->cache_key );

            if( false === $remote || ! $this->cache_allowed ) {

                $remote = wp_remote_get(
                    $this->plugin_sitelocation . 'info.json',
                    array(
                        //'sslverify' => false,
                        'timeout' => 10,
                        'headers' => array(
                            'Accept' => 'application/json'
                        )
                    )
                );

                if(
                    is_wp_error( $remote )
                    || 200 !== wp_remote_retrieve_response_code( $remote )
                    || empty( wp_remote_retrieve_body( $remote ) )
                ) {
                    return false;
                }

                set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );

            }

            $remote = json_decode( wp_remote_retrieve_body( $remote ) );

            return $remote;

        }


        function info( $res, $action, $args ) {

            // print_r( $action );
            // print_r( $args );

            // do nothing if you're not getting plugin information right now
            if( 'plugin_information' !== $action ) {
                return false;
            }

            // do nothing if it is not our plugin
            if( $this->plugin_slug !== $args->slug ) {
                return false;
            }

            // get updates
            $remote = $this->request();

            if( ! $remote ) {
                return false;
            }

            $res = new stdClass();

            $res->name = $remote->name;
            $res->slug = $remote->slug;
            $res->version = $remote->version;
            $res->tested = $remote->tested;
            $res->requires = $remote->requires;
            $res->author = $remote->author;
            $res->author_profile = $remote->author_profile;
            $res->download_link = $remote->download_url;
            $res->trunk = $remote->download_url;
            $res->requires_php = $remote->requires_php;
            $res->last_updated = $remote->last_updated;

            $res->sections = array(
                'description' => $remote->sections->description,
                'installation' => $remote->sections->installation,
                'changelog' => $remote->sections->changelog
            );

            if( ! empty( $remote->banners ) ) {
                $res->banners = array(
                    'low' => $remote->banners->low,
                    'high' => $remote->banners->high
                );
            }

            return $res;

        }

        public function update( $transient ) {

            if ( empty($transient->checked ) ) {
                return $transient;
            }

            $remote = $this->request();

            if (
                $remote
                && version_compare( $this->version, $remote->version, '<' )
                && version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
                && version_compare( $remote->requires_php, PHP_VERSION, '<' )
            ) {
                $res = new stdClass();
                $res->slug = $this->plugin_slug;
                $res->plugin = $this->plugin_filename; // worthstock/worthstock.php
                $res->new_version = $remote->version;
                $res->tested = $remote->tested;
                $res->package = $remote->download_url;

                $transient->response[ $res->plugin ] = $res;
            } else {
                $res = new stdClass();
                $res->slug = $this->plugin_slug;
                $res->plugin = $this->plugin_filename; // worthstock/worthstock.php
                $res->new_version = $this->version;
                $res->tested = '';
                $res->package = '';
                // Adding the "mock" item to the `no_update` property is required
                // for the enable/disable auto-updates links to correctly appear in UI.
                $transient->no_update['my-plugin/my-plugin.php'] = $res;
                
            }

            return $transient;

        }

        public function purge( $upgrader_object, $options ) {

            if (
                $this->cache_allowed
                && 'update' === $options['action']
                && 'plugin' === $options[ 'type' ]
                && isset( $options['plugins'])
            ) {
                foreach( $options['plugins'] as $plugin ) {
                    if( $plugin == $this->plugin_filename ) {
                        // just clean the cache when new plugin version is installed
                        delete_transient( $this->cache_key );
                    }
                }
            }

        }


    }

}