<?php
/*
Plugin Name: Plug-In WORTHSTOCK

Description: Connettore API per schede prodotto
Version: 1.0.8.21
Author: WORTHSTOCK
Author URI: https://www.worthstock.it/
License: GPLv2 or later
Text Domain: worthstock
Domain Path /languages

WC requires at least: 3.2
WC tested up to: 5.8
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

define( 'WORTHSTOCK__PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
define( 'WORTHSTOCK__PLUGIN_URL', plugin_dir_url(__FILE__) );
define( 'WORTHSTOCK__DEBUG', true);
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.base.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.sync.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.receiver.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.datamapper.php' );
//if ( !class_exists('Google\Site_Kit\Core\Storage\Data_Encryption') ) {
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.encryption.php' );
//}
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.enqueue.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.frontend.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.settings.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/classes/class.updater.php' );
require_once( WORTHSTOCK__PLUGIN_DIR . '/samples/sampledata.php' );

/**
 * E' la classe di attivazione del plugin, di inclusione del codice delle diverse sottoclassi, di creazione del menu admin del plugin e di registrazione degli hook e dei filtri.
 */
final class Worthstock {

    /**
	 * @var Worthstock he himself (in case we're going to implement singlenton) 
	 */
	private static $_instance = null;

	/**
	 * plugin version
	 */
	public static $plugin_version = '1.0.8.21';

	/**
	 * plugin dirname
	 */
	public $plugin_slug;
    
	/**
	 * plugin filename
	 */
	public $plugin_filename;

	/**
	 * @var WorthstockFrontend
	 */
	public $ui;

    /**
     * @var WorthstockSettings
     */
	public $settings;

	/**
	 * @var WorthstockSync
	 */
	public $sync;

	/**
	 * @var WorthstockDataMapper
	 */
	public $mapper;

	/**
	 * @var WorthstockReceiver
	 */
	public $receiver;

	/**
	 * @var WorthstockUpdater
	 */
	public $updater;

    /**
     * @var bool
     */
    private $enable_woocommerce_replication;
    
    /**
     * @var string
     */
    public $default_language_code;

    
	/**
	 * Load text domain
	 */
	public function load_textdomain($domain='worthstock') {
		//load_plugin_textdomain( 'worthstock' );
        // Load from plugin lang folder.
        load_textdomain( $domain, WORTHSTOCK__PLUGIN_DIR .  '/languages/worthstock-it_IT.mo' );
	}

    /**
     * get site locale short code
     */
    public function get_wordpress_language() {
        $currentLanguage = get_bloginfo('language');
        $currentLanguage = ($currentLanguage ? $currentLanguage : get_locale());
        $currentLanguage = ($currentLanguage ? $currentLanguage : 'it');

        return strtoupper(substr($currentLanguage, 0, 2));
    }

	/**
	 * Construct
	 */
	public function __construct() {
		global $wpdb;
        $this->plugin_filename = plugin_basename( __FILE__ );
        $this->plugin_slug = plugin_basename( __DIR__ );

		$this->ui            = new WorthstockFrontend();
		$this->settings      = new WorthstockSettings();
		$this->sync           = new WorthstockSync();
		$this->receiver           = new WorthstockReceiver();
		$this->queue = new WorthstockEnqueue();
		$this->mapper = new WorthstockDataMapper();
		$this->update = new WorthstockUpdater();
        $this->enable_woocommerce_replication = true;
        $this->default_language_code = $this->get_wordpress_language();
		
		add_action( 'plugins_loaded', array( &$this, 'load_textdomain' ) );
		add_action( 'init', array( &$this, 'init' ) );
	}    

	/**
	 * @return Worthstock
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) )
			self::$_instance = new Worthstock();
		return self::$_instance;
	}

    /**
     * check dependencies
     */
    private function check_dependencies() {
        if ( ! $this->worthstock_is_woocommerce_activated() ) 
        {
            add_action( 'admin_notices', array($this, 'worthstock_dependencies_admin_notice') );
            return;
        }
    }

    /**
     * toggle woocommerce_replication
     */
    public function set_woocommerce_replication($flag) {
        if (is_bool($flag)) {
            $this->enable_woocommerce_replication = $flag;
        }
    }

    /**
     * read woocommerce_replication toggle
     */
    public function get_woocommerce_replication() {
        return $this->enable_woocommerce_replication;
    }
    
	public function init() {
        $this->register_plugin();
        $this->check_dependencies();
        $this->activate_hooks();
        $this->define_worthstock_hooks();
        $this->woocommerce_events_hooks();
	}

    private function register_plugin() {
        register_activation_hook( __FILE__, array( 'worthstock', 'plugin_activation' ) );
        register_deactivation_hook( __FILE__, array( 'worthstock', 'plugin_deactivation' ) );
        register_uninstall_hook( __FILE__, array( 'worthstock', 'plugin_uninstall' ) );
    }
    
    private function define_worthstock_hooks() {
        add_action( 'worthstock_login', array($this->sync,'api_login') );
        add_action( 'worthstock_api_get_single_product', array($this->sync,'api_get_single_product'),10,2 );
        add_action( 'worthstock_api_get_all_products', array($this->sync,'api_get_all_products'),10,1 );
        add_action( 'worthstock_api_get_categories', array($this->sync,'api_get_categories'),10,1 );
        add_action( 'worthstock_api_put_prices', array($this->sync,'remote_update_prices'),10,3 );
        add_action( 'worthstock_api_put_quantity', array($this->sync,'remote_update_quantity'),10,3 );
        add_action( 'worthstock_api_put_product', array($this->sync,'remote_update_product'),10,3 );
        add_action( 'worthstock_api_post_product', array($this->sync,'remote_insert_product'),10,3 );
        add_action( 'worthstock_api_delete_product', array($this->sync,'remote_delete_product'),10,2 );
    }

    private function activate_hooks() {
        add_action( 'woocommerce_product_options_sku', array($this->ui,'product_print_custom_meta'),10,1 );
        add_action( 'woocommerce_process_product_meta', array($this->ui,'product_save_custom_meta'),10,1 );
        add_action( 'woocommerce_product_after_variable_attributes', array($this->ui,'variation_print_custom_meta'), 10, 3 );
        //add_action( 'woocommerce_save_product_variation', array($this->ui,'variation_save_custom_meta'), 10, 2 );
        add_filter( 'woocommerce_available_variation', array($this->ui,'variation_load_custom_meta') );
        // Adding a custom Meta container to admin products pages
        add_action( 'add_meta_boxes', array($this->ui,'product_custom_meta_box') );

        // categories
        add_action( 'product_cat_add_form_fields', array($this->sync,'pos_new_cat_meta_field'), 10, 2 );
        add_action( 'product_cat_edit_form_fields', array($this->sync,'pos_edit_cat_meta_field'), 10, 2 );
        add_action( 'edited_product_cat', array($this->sync,'save_pos_cat_custom_meta'), 10, 1 );
        add_action( 'create_product_cat', array($this->sync,'save_pos_cat_custom_meta'), 10, 1 );
        // admin pages
        add_action('admin_menu', array($this, 'worthstock_in_menu'));
        // enqueue scripts
        //add_action('admin_enqueue_scripts', array($this, 'worthstock_import_script_and_style'));
    
        /**
         * AJAX
         */
        add_action('wp_ajax_api_actions', array($this->ui, 'ajax_do_action'), 10); // wp_ajax_{action}
        add_action('wp_ajax_nopriv_api_actions', array($this->ui, 'ajax_do_action'), 10); // wp_ajax_nopriv_{action}

        
        //Hooks
        add_action( 'current_screen', array($this->ui, 'product_custom_bulk_hooks') );

        //languages
        //add_filter( 'load_textdomain_mofile', array($this, 'plugin_load_textdomain'), 10, 2 );

        //Woocommerce Duplicate post. TODO: generalize this
        add_filter( 'woocommerce_duplicate_product_exclude_meta', array($this, 'worthstock_duplicate_meta_fields_filter'), 10, 1 );

    }
    
    /**
     * woocommerce events that triggers a remote notification
     */
    private function woocommerce_events_hooks() {
        if ($this->enable_woocommerce_replication !== true) return false;
        // product update / creation
        // new order (?)
        // stock change
        add_action('woocommerce_product_set_stock', array($this,'add_hooked_stock_event'));
        add_action('woocommerce_updated_product_stock', array($this,'add_hooked_stock_event'));
        add_action('woocommerce_variation_set_stock', array($this,'add_hooked_stock_event'));

        // price change
        add_action('woocommerce_updated_product_price', array($this,'add_hooked_price_event'));

        // save serialized wrapper (spostato dopo il save_worthstock_custom_product_meta)
        // add_action('pre_post_update', array($this->queue,'save_serialized_product'), 10, 2);
        // On admin single product creation/update
        add_action( 'woocommerce_update_product', array($this,'add_hooked_update_event') );
        add_action( 'woocommerce_new_product', array($this,'add_hooked_insert_event') );
        
        // On admin single product variation creation/update
        // add_action( 'woocommerce_save_product_variation', array($this->queue,'notify_variation_updated') );

        // product deleted
        add_action( 'wp_trash_post', array($this,'add_hooked_delete_event'), 99 );
        add_action( 'delete_post', array($this,'add_hooked_delete_event'), 99 );

        add_action( 'shutdown', array($this->queue,'resolve_queue') );
    }

    /** wrappers to real function in enqueue class */
    public function add_hooked_event($event_type, $product_id) {
        $this->queue->add_event_to_queue($event_type, $product_id, 'hook');
    }
    public function add_hooked_insert_event($product_id) {
        $this->add_hooked_event('insert', $product_id, 'hook');
    }
    public function add_hooked_update_event($product_id) {
        $this->add_hooked_event('update', $product_id, 'hook');
    }
    public function add_hooked_stock_event($product_id) {
        $this->add_hooked_event('stock', $product_id, 'hook');
    }
    public function add_hooked_price_event($product_id) {
        $this->add_hooked_event('price', $product_id, 'hook');
    }
    public function add_hooked_delete_event($product_id) {
        $this->add_hooked_event('delete', $product_id, 'hook');
    }    

	/**
	 * Attached to activate_{ plugin_basename( __FILES__ ) } by register_activation_hook()
	 * @static
	 */
	public function plugin_activation() {
        //$this->convertWorthstockLegacyCustomParam();
		return true;
	}

    /**
     * Attached to deactivate_{ plugin_basename( __FILES__ ) } by register_deactivation_hook()
     * @static
     */
	public static function plugin_deactivation( ) {
        return true;
	}
    /**
     * Attached to uninstall_{ plugin_basename( __FILES__ ) } by register_uninstall_hook()
     * @static
     */

	public static function plugin_uninstall() {
        if ( ! current_user_can( 'activate_plugins' ) )
            return;
        
        $worthstock_obj = Worthstock::instance();
            if ( is_null($worthstock_obj) ) return;

        if ($worthstock_obj->settings->get_option( 'worthstock_remove_all_data' ) == 'yes' ) {
            // suspend remote product remuval (stucks in case of +1000 products)
            // $worthstock_obj->sync->delete_remote_products(true);
            $worthstock_obj->sync->remove_product_metadata(0, true);
            // blank api login call to remove webhookurl
            $ret = $worthstock_obj->sync->api_login(false, array("setWebhookUrl"=>false));
            // remove plugin settings
            delete_option( 'worthstock-settings');
        }

        // remove token transient
        delete_transient('_worthstock_token');

        # Uncomment the following line to see the function in action
        //exit( var_dump( $_GET ) );
    }

    public function plugin_load_textdomain( $mofile, $domain ) {
        if ( 'worthstock' === $domain && false !== strpos( $mofile, WP_LANG_DIR . '/plugins/' ) ) {
            $locale = apply_filters( 'plugin_locale', determine_locale(), $domain );
            $mofile = WP_PLUGIN_DIR . '/' . dirname( plugin_basename( __FILE__ ) ) . '/languages/' . $domain . '-' . $locale . '.mo';
        }
        return $mofile;
    }


    /**
     * create menu
     */
    function worthstock_in_menu()
    {
        if ( $this->worthstock_check_capabilities() ) {
            $hook = add_menu_page('WORTHSTOCK', 'WORTHSTOCK', 'edit_others_posts', 'worthstock', array($this, 'worthstock_show_main_page'));
            add_submenu_page('worthstock', 'WORTHSTOCK', 'Home', 'edit_others_posts', 'worthstock');
                //add_submenu_page('worthstock', 'Get Date Interval', 'Get Date Interval', 'edit_others_posts', 'get_date_interval', 'worthstock_show_date_interval_page', $position = 1);
            add_action( 'load-' . $hook, array($this, 'worthstock_import_script_and_style') );
            // register scripts & styles, specific for the settings page
            //add_action( "admin_print_scripts-{$this->hook}", array( $this, 'worthstock_import_script_and_style' ) );

            $sub_hook_setting = add_submenu_page(
                'worthstock',
                __( 'Worthstock plugin Settings', 'worthstock' ), 	// <title> tag
                __( 'Impostazioni', 'worthstock' ), 			// menu label
                'manage_options', 								// required cap to view this page
                'worthstock-setting-page', 			// page slug
                array( &$this->settings, 'display_settings_page' ),			// callback
                2
            );
            add_action( 'load-' . $sub_hook_setting, array($this, 'worthstock_import_script_and_style') );
            $sub_hook = add_submenu_page(
                'worthstock',
                __( 'Worthstock help manual', 'worthstock' ), 	// <title> tag
                __( 'Aiuto', 'worthstock' ), 			// menu label
                'manage_options', 								// required cap to view this page
                'worthstock-help-page', 			// page slug
                array($this, 'worthstock_show_help_page'),		// callback
                3
            );
            add_action( 'load-' . $sub_hook, array($this, 'worthstock_import_script_and_style') );
        }
    }

    /**
     * check if user has the right permissions
     */
    function worthstock_check_capabilities()
    {
        return current_user_can( 'manage_woocommerce' );
    }


    function worthstock_is_woocommerce_activated() {
        return class_exists( 'woocommerce' );
    }


    /**
     * show main page
     */
    function worthstock_show_main_page()
    {
        $count_synced_products = $this->sync->countProductsByWorthstockId();
        //$count_synced_products = 0;
        require_once(__DIR__ . '/pages/main.php');
    }
    
    /**
     * show help page
     */
    function worthstock_show_help_page()
    {
        require_once(__DIR__ . '/pages/help.php');
    }

    /**
     * show date_interval page
     */
    function worthstock_show_date_interval_page()
    {
        require_once(__DIR__ . '/pages/date_interval.php');
    }


    /**
     * import bootstrap 5 + icons
     */
    function worthstock_import_script_and_style($hook)
    {
        if ( !is_admin() ) { return; }

        wp_enqueue_style('bootstrap-css', "https://cdn.jsdelivr.net/npm/bootstrap@5.1.1/dist/css/bootstrap.min.css");
        wp_enqueue_style('bootstrap-icons', "https://cdn.jsdelivr.net/npm/bootstrap-icons@1.5.0/font/bootstrap-icons.css");
        wp_enqueue_script('bootstrap-js', "https://cdn.jsdelivr.net/npm/bootstrap@5.1.1/dist/js/bootstrap.bundle.min.js");
        wp_enqueue_style( 'worthstock-style', plugin_dir_url(__FILE__) . "inc/style.css");
        wp_enqueue_script( 'worthstock-script', plugin_dir_url(__FILE__) . "inc/worthstock.js");

        wp_localize_script('worthstock-script', 'worthstock_params', array(
            'ajaxurl' => admin_url('admin-ajax.php'), // WordPress AJAX
            'current_page' => get_query_var('paged') ? get_query_var('paged') : 1,
            'nonce' => wp_create_nonce('ws-nonce')
        ));
    }


    /**
     *  display custom admin notice
     **/
    function worthstock_dependencies_admin_notice() {
        
        $screen = get_current_screen();
        ?>
                
        <?php
        
        if ($screen->id === 'toplevel_page_worthstock') {
                ?>  
                <div class="notice notice-error is-dismissible">
                    <p><?php _e('Il plugin Worthstock ha bisogno che Woocommerce sia installato ed attivo nel sito.', 'worthstock'); ?></p>
                </div>
            <?php
        }
    }

    /** 
     * Woocommerce Duplicate product: Filters out worthstock custom fields from being duplicated in addition to the defaults.
     *
     * @param array $meta_excludelist The default exclusion list, based on the “Do not copy these fields” setting, plus some other field names.
     *
     * @return array The custom fields to exclude.
     */
    function worthstock_duplicate_meta_fields_filter( $meta_excludelist ) {
        // Merges the defaults array with our own array of custom fields.
        $worthstock_custom_metas = [];
        $params = $this->mapper->getWorthstockCustomParamList();
        $params = array_merge($params, $this->mapper->getWorthstockCustomParamList('variant'));
        foreach($params as $key => $value) {
            $worthstock_custom_metas[] = $this->mapper->getWorthstockCustomParamName($value);
        }
        array_push($worthstock_custom_metas,'_worthstock_last_operation');
        return array_merge( $meta_excludelist, $worthstock_custom_metas );
    }    

    function convertWorthstockLegacyCustomParam() {
        global $wpdb;
        $params = $this->mapper->getWorthstockCustomParamList();
        $params = array_merge($params, $this->mapper->getWorthstockCustomParamList('variant'));
        
        foreach($params as $key => $field) {
            $new_field_name = $this->mapper->getWorthstockCustomParamName($field);
            $old_field_name =  "_" . $field;
            $result = $wpdb->query( $wpdb->prepare( "UPDATE $wpdb->postmeta SET meta_key = '%s' WHERE meta_key = '%s'", $new_field_name, $old_field_name ) );            
        }
    }
}

//$worthstock_obj = new Worthstock();

$worthstock_obj = Worthstock::instance();